@@ -32,7 +32,10 @@ from openpyxl.xml.constants import (
     CHART_TYPE,
     CHARTSHAPE_TYPE,
     CHARTSHEET_TYPE,
-    CONTYPES_NS
+    CONTYPES_NS,
+    ACTIVEX,
+    CTRL,
+    VBA,
 )
 from openpyxl.xml.functions import tostring
 
@@ -76,7 +79,6 @@ DEFAULT_TYPES = [
 ]
 
 DEFAULT_OVERRIDE = [
-    Override("/" + ARC_SHARED_STRINGS, SHARED_STRINGS), # Shared strings
     Override("/" + ARC_STYLE, STYLES_TYPE), # Styles
     Override("/" + ARC_THEME, THEME_TYPE), # Theme
     Override("/docProps/core.xml", "application/vnd.openxmlformats-package.core-properties+xml"),
@@ -204,5 +206,7 @@ class Manifest(Serialisable):
             mf = Manifest.from_tree(node)
             filenames = self.filenames
             for override in mf.Override:
+                if override.PartName not in (ACTIVEX, CTRL, VBA):
+                    continue
                 if override.PartName not in filenames:
                     self.Override.append(override)
