@@ -1,52 +1,38 @@
 from __future__ import absolute_import
-# Copyright (c) 2010-2016 openpyxl
+# Copyright (c) 2010-2019 openpyxl
 
 """Worksheet is the 2nd-level container in Excel."""
 
 
 # Python stdlib imports
 from itertools import islice, product
-import re
+from operator import itemgetter
 from inspect import isgenerator
-from warnings import warn
 
 # compatibility imports
 from openpyxl.compat import (
-    unicode,
-    range,
     basestring,
     deprecated,
-    safe_string
 )
+try:
+    range = xrange
+except NameError:
+    pass
 
 # package imports
 from openpyxl.utils import (
-    coordinate_from_string,
     column_index_from_string,
     get_column_letter,
     range_boundaries,
-    rows_from_range,
     coordinate_to_tuple,
     absolute_coordinate,
 )
-from openpyxl.utils.cell import COORD_RE
-
-from openpyxl.cell import Cell
-from openpyxl.utils.exceptions import (
-    SheetTitleException,
-    InsufficientCoordinatesException,
-    NamedRangeException
-)
-from openpyxl.utils.units import (
-    points_to_pixels,
-    DEFAULT_COLUMN_WIDTH,
-    DEFAULT_ROW_HEIGHT,
-)
+from openpyxl.cell import Cell, MergedCell
 from openpyxl.formatting.formatting import ConditionalFormattingList
 from openpyxl.packaging.relationship import RelationshipList
 from openpyxl.workbook.child import _WorkbookChild
 from openpyxl.workbook.defined_name import COL_RANGE_RE, ROW_RANGE_RE
-from openpyxl.utils.bound_dictionary import BoundDictionary
+from openpyxl.formula.translate import Translator
 
 from .datavalidation import DataValidationList
 from .page import (
@@ -61,23 +47,17 @@ from .dimensions import (
     SheetFormatProperties,
 )
 from .protection import SheetProtection
-from .filters import AutoFilter, SortState
+from .filters import AutoFilter
 from .views import (
-    SheetView,
     Pane,
     Selection,
     SheetViewList,
 )
+from .cell_range import MultiCellRange, CellRange
+from .merge import MergedCellRange
 from .properties import WorksheetProperties
-from .pagebreak import PageBreak
-
-
-@deprecated("Use the worksheet.values property")
-def flatten(results):
-    """Return cell values row-by-row"""
-
-    for row in results:
-        yield(c.value for c in row)
+from .pagebreak import RowBreak, ColBreak
+from .scenario import ScenarioList
 
 
 class Worksheet(_WorkbookChild):
@@ -122,18 +102,22 @@ class Worksheet(_WorkbookChild):
         self._setup()
 
     def _setup(self):
-        self.row_dimensions = BoundDictionary("index", self._add_row)
+        self.row_dimensions = DimensionHolder(worksheet=self,
+                                              default_factory=self._add_row)
         self.column_dimensions = DimensionHolder(worksheet=self,
                                                  default_factory=self._add_column)
-        self.page_breaks = PageBreak()
+        self.row_breaks = RowBreak()
+        self.col_breaks = ColBreak()
+        self.page_breaks = [self.row_breaks, self.col_breaks]
         self._cells = {}
         self._charts = []
         self._images = []
         self._rels = RelationshipList()
         self._drawing = None
         self._comments = []
-        self._merged_cells = []
+        self.merged_cells = MultiCellRange()
         self._tables = []
+        self._pivots = []
         self.data_validations = DataValidationList()
         self._hyperlinks = []
         self.sheet_state = 'visible'
@@ -148,7 +132,6 @@ class Worksheet(_WorkbookChild):
 
         self._current_row = 0
         self.auto_filter = AutoFilter()
-        self.sort_state = SortState()
         self.paper_size = None
         self.formula_attributes = {}
         self.orientation = None
@@ -156,6 +139,8 @@ class Worksheet(_WorkbookChild):
         self.legacy_drawing = None
         self.sheet_properties = WorksheetProperties()
         self.sheet_format = SheetFormatProperties()
+        self._merged_cell_range = {}
+        self.scenarios = ScenarioList()
 
 
     @property
@@ -165,12 +150,12 @@ class Worksheet(_WorkbookChild):
 
     @property
     def selected_cell(self):
-        return self.sheet_view.selection.sqref
+        return self.sheet_view.selection[0].sqref
 
 
     @property
     def active_cell(self):
-        return self.sheet_view.selection.activeCell
+        return self.sheet_view.selection[0].activeCell
 
     @property
     def show_gridlines(self):
@@ -186,31 +171,6 @@ class Worksheet(_WorkbookChild):
     def show_summary_right(self):
         return self.sheet_properties.outlinePr.summaryRight
 
-    @property
-    def vba_code(self):
-        for attr in ("codeName", "enableFormatConditionsCalculation",
-                     "filterMode", "published", "syncHorizontal", "syncRef",
-                     "syncVertical", "transitionEvaluation", "transitionEntry"):
-            value = getattr(self.sheet_properties, attr)
-            if value is not None:
-                yield attr, safe_string(value)
-
-    @vba_code.setter
-    def vba_code(self, value):
-        for k, v in value.items():
-            if k in ("codeName", "enableFormatConditionsCalculation",
-                     "filterMode", "published", "syncHorizontal", "syncRef",
-                     "syncVertical", "transitionEvaluation", "transitionEntry"):
-                setattr(self.sheet_properties, k, v)
-
-    """ End To keep compatibility with previous versions"""
-
-
-    @deprecated("Use the ws.values property")
-    def get_cell_collection(self):
-        """Return an unordered list of the cells in this worksheet."""
-        return self._cells.values()
-
 
     @property
     def freeze_panes(self):
@@ -253,23 +213,9 @@ class Worksheet(_WorkbookChild):
             view.selection = sel
 
 
-    @deprecated("Set print titles rows or columns directly")
-    def add_print_title(self, n, rows_or_cols='rows'):
-        """ Print Titles are rows or columns that are repeated on each printed sheet.
-        This adds n rows or columns at the top or left of the sheet
+    def cell(self, row, column, value=None):
         """
-
-        scope = self.parent.get_index(self)
-
-        if rows_or_cols == 'cols':
-            self.print_title_cols = 'A:%s' % get_column_letter(n)
-
-        else:
-            self.print_title_rows = '1:%d' % n
-
-
-    def cell(self, coordinate=None, row=None, column=None, value=None):
-        """Returns a cell object based on the given coordinates.
+        Returns a cell object based on the given coordinates.
 
         Usage: cell(row=15, column=1, value=5)
 
@@ -282,27 +228,12 @@ class Worksheet(_WorkbookChild):
         :param column: column index of the cell (e.g. 3)
         :type column: int
 
-        :param coordinate: coordinates of the cell (e.g. 'B12')
-        :type coordinate: string
-
         :param value: value of the cell (e.g. 5)
         :type value: numeric or time or string or bool or none
 
-        :raise: InsufficientCoordinatesException when neither row nor column are not given
-
-        :rtype: openpyxl.cell.Cell
-
+        :rtype: openpyxl.cell.cell.Cell
         """
 
-        if (row is None or column is None) and coordinate is None:
-            msg = "You have to provide a value either for " \
-                   "'coordinate' or for 'row' *and* 'column'"
-            raise InsufficientCoordinatesException(msg)
-
-        if coordinate is not None:
-            warn("Using a coordinate with ws.cell is deprecated. Use ws[coordinate] instead")
-            row, column = coordinate_to_tuple(coordinate)
-
         if row < 1 or column < 1:
             raise ValueError("Row or column values must be at least 1")
 
@@ -320,7 +251,7 @@ class Worksheet(_WorkbookChild):
         """
         coordinate = (row, column)
         if not coordinate in self._cells:
-            cell = Cell(self, row=row, col_idx=column)
+            cell = Cell(self, row=row, column=column)
             self._add_cell(cell)
         return self._cells[coordinate]
 
@@ -384,8 +315,18 @@ class Worksheet(_WorkbookChild):
         return self.iter_rows()
 
 
+    def __delitem__(self, key):
+        row, column = coordinate_to_tuple(key)
+        if (row, column) in self._cells:
+            del self._cells[(row, column)]
+
+
     @property
     def min_row(self):
+        """The minimium row index containing data (1-based)
+
+        :type: int
+        """
         min_row = 1
         if self._cells:
             rows = set(c[0] for c in self._cells)
@@ -395,9 +336,9 @@ class Worksheet(_WorkbookChild):
 
     @property
     def max_row(self):
-        """Returns the maximum row index containing data
+        """The maximum row index containing data (1-based)
 
-        :rtype: int
+        :type: int
         """
         max_row = 1
         if self._cells:
@@ -408,6 +349,10 @@ class Worksheet(_WorkbookChild):
 
     @property
     def min_column(self):
+        """The minimum column index containing data (1-based)
+
+        :type: int
+        """
         min_col = 1
         if self._cells:
             cols = set(c[1] for c in self._cells)
@@ -417,9 +362,9 @@ class Worksheet(_WorkbookChild):
 
     @property
     def max_column(self):
-        """Get the largest value for column currently stored.
+        """The maximum column index containing data (1-based)
 
-        :rtype: int
+        :type: int
         """
         max_col = 1
         if self._cells:
@@ -429,7 +374,10 @@ class Worksheet(_WorkbookChild):
 
 
     def calculate_dimension(self):
-        """Return the minimum bounding range for all cells containing data."""
+        """Return the minimum bounding range for all cells containing data (ex. 'A1:M24')
+
+        :rtype: string
+        """
         if self._cells:
             rows = set()
             cols = set()
@@ -451,25 +399,19 @@ class Worksheet(_WorkbookChild):
 
     @property
     def dimensions(self):
+        """Returns the result of :func:`calculate_dimension`"""
         return self.calculate_dimension()
 
 
-    def iter_rows(self, range_string=None, min_row=None, max_row=None, min_col=None, max_col=None,
-                  row_offset=0, column_offset=0):
+    def iter_rows(self, min_row=None, max_row=None, min_col=None, max_col=None, values_only=False):
         """
-        Return cells from the worksheet as rows. Boundaries for the cells can
-        be passed in either as indices of rows and columns.
+        Produces cells from the worksheet, by row. Specify the iteration range
+        using indices of rows and columns.
 
-        If no boundaries are passed in the cells will start at A1.
+        If no indices are specified the range starts at A1.
 
         If no cells are in the worksheet an empty tuple will be returned.
 
-
-        Additional rows and columns can be created using offsets.
-
-        :param range_string: range string (e.g. 'A1:B2') *deprecated*
-        :type range_string: string
-
         :param min_col: smallest column index (1-based index)
         :type min_col: int
 
@@ -479,22 +421,15 @@ class Worksheet(_WorkbookChild):
         :param max_col: largest column index (1-based index)
         :type max_col: int
 
-        :param max_row: smallest row index (1-based index)
+        :param max_row: largest row index (1-based index)
         :type max_row: int
 
-        :param row_offset: additional rows (e.g. 4)
-        :type row_offset: int
-
-        :param column_offset: additional columns (e.g. 3)
-        :type column_offset: int
+        :param values_only: whether only cell values should be returned
+        :type values_only: bool
 
         :rtype: generator
         """
 
-        if range_string is not None:
-            warn("Using a range string is deprecated. Use ws[range_string]")
-            min_col, min_row, max_col, max_row = range_boundaries(range_string.upper())
-
         if self._current_row == 0 and not any([min_col, min_row, max_col, max_row ]):
             return ()
 
@@ -503,42 +438,43 @@ class Worksheet(_WorkbookChild):
         max_col = max_col or self.max_column
         max_row = max_row or self.max_row
 
-        if max_col is not None:
-            max_col += column_offset
-        if max_row is not None:
-            max_row += row_offset
-        return self._cells_by_row(min_col + column_offset,
-                                  min_row + row_offset,
-                                  max_col,
-                                  max_row)
+        return self._cells_by_row(min_col, min_row, max_col, max_row, values_only)
 
 
-    def _cells_by_row(self, min_col, min_row, max_col, max_row):
+    def _cells_by_row(self, min_col, min_row, max_col, max_row, values_only=False):
         for row in range(min_row, max_row + 1):
-            yield tuple(self.cell(row=row, column=column)
-                    for column in range(min_col, max_col + 1))
+            cells = (self.cell(row=row, column=column) for column in range(min_col, max_col + 1))
+            if values_only:
+                yield tuple(cell.value for cell in cells)
+            else:
+                yield tuple(cells)
 
 
     @property
     def rows(self):
-        """Iterate over all rows in the worksheet"""
+        """Produces all cells in the worksheet, by row (see :func:`iter_rows`)
+
+        :type: generator
+        """
         return self.iter_rows()
 
 
     @property
     def values(self):
+        """Produces all cell values in the worksheet, by row
+
+        :type: generator
         """
-        Return the values of all the cells, row by row
-        """
-        for row in self.iter_rows():
-            yield tuple(c.value for c in row)
+        for row in self.iter_rows(values_only=True):
+            yield row
 
 
-    def iter_cols(self, min_col=None, max_col=None, min_row=None, max_row=None):
+    def iter_cols(self, min_col=None, max_col=None, min_row=None, max_row=None, values_only=False):
         """
-        Returns all cells in the worksheet from the first row as columns.
+        Produces cells from the worksheet, by column. Specify the iteration range
+        using indices of rows and columns.
 
-        If no boundaries are passed in the cells will start at A1.
+        If no indices are specified the range starts at A1.
 
         If no cells are in the worksheet an empty tuple will be returned.
 
@@ -551,13 +487,16 @@ class Worksheet(_WorkbookChild):
         :param max_col: largest column index (1-based index)
         :type max_col: int
 
-        :param max_row: smallest row index (1-based index)
+        :param max_row: largest row index (1-based index)
         :type max_row: int
 
+        :param values_only: whether only cell values should be returned
+        :type values_only: bool
+
         :rtype: generator
         """
 
-        if self._current_row == 0 and not any([min_col, min_row, max_col, max_row ]):
+        if self._current_row == 0 and not any([min_col, min_row, max_col, max_row]):
             return ()
 
         min_col = min_col or 1
@@ -565,93 +504,32 @@ class Worksheet(_WorkbookChild):
         max_col = max_col or self.max_column
         max_row = max_row or self.max_row
 
-        return self._cells_by_col(
-            min_col, min_row, max_col, max_row
-        )
+        return self._cells_by_col(min_col, min_row, max_col, max_row, values_only)
 
 
-    def _cells_by_col(self, min_col, min_row, max_col, max_row):
+    def _cells_by_col(self, min_col, min_row, max_col, max_row, values_only=False):
         """
         Get cells by column
         """
         for column in range(min_col, max_col+1):
-            yield tuple(self.cell(row=row, column=column)
+            cells = (self.cell(row=row, column=column)
                         for row in range(min_row, max_row+1))
+            if values_only:
+                yield tuple(cell.value for cell in cells)
+            else:
+                yield tuple(cells)
 
 
     @property
     def columns(self):
-        """Iterate over all columns in the worksheet"""
+        """Produces all cells in the worksheet, by column  (see :func:`iter_cols`)"""
         return self.iter_cols()
 
 
-    @deprecated("""
-    Use ws.iter_rows() or ws.iter_cols() depending whether you
-    want rows or columns returned.
-    """)
-    def get_squared_range(self, min_col, min_row, max_col, max_row):
-        """Returns a 2D array of cells. Will create any cells within the
-        boundaries that do not already exist
-
-        :param min_col: smallest column index (1-based index)
-        :type min_col: int
-
-        :param min_row: smallest row index (1-based index)
-        :type min_row: int
-
-        :param max_col: largest column index (1-based index)
-        :type max_col: int
-
-        :param max_row: smallest row index (1-based index)
-        :type max_row: int
-
-        :rtype: generator
-        """
-
-        return self._cells_by_row(min_col, min_row, max_col, max_row)
-
-
-    @deprecated("""Ranges are workbook objects. Use wb.defined_names[range_name]""")
-    def get_named_range(self, range_name):
-        """
-        Returns a 2D array of cells, with optional row and column offsets.
-
-        :param range_name: `named range` name
-        :type range_name: string
-
-        :rtype: tuple[tuple[openpyxl.cell.Cell]]
-        """
-        defn = self.parent.defined_names[range_name]
-        if defn.localSheetId and defn.localSheetId != self.parent.get_index(self):
-            msg = "{0} not available in this worksheet".format(range_name)
-            raise KeyError(msg)
-
-        if defn.type != "RANGE":
-            msg = '{0} refers to a value, not a range'.format(range_name)
-            raise NameError(msg)
-
-        result = []
-        for title, cells_range in defn.destinations:
-            ws = self.parent[title]
-            if ws != self:
-                raise NamedRangeException("Range includes cells from another worksheet")
-
-            rows = ws[cells_range]
-            if isinstance(rows, Cell):
-                rows = [(rows, )]
-
-            for row in rows:
-                result.extend(row)
-
-        return tuple(result)
-
-
     def set_printer_settings(self, paper_size, orientation):
         """Set printer settings """
 
         self.page_setup.paperSize = paper_size
-        if orientation not in (self.ORIENTATION_PORTRAIT, self.ORIENTATION_LANDSCAPE):
-            raise ValueError("Values should be %s or %s" % (self.ORIENTATION_PORTRAIT, self.ORIENTATION_LANDSCAPE))
         self.page_setup.orientation = orientation
 
 
@@ -662,6 +540,7 @@ class Worksheet(_WorkbookChild):
         """
         self.data_validations.append(data_validation)
 
+
     def add_chart(self, chart, anchor=None):
         """
         Add a chart to the sheet
@@ -671,14 +550,14 @@ class Worksheet(_WorkbookChild):
             chart.anchor = anchor
         self._charts.append(chart)
 
+
     def add_image(self, img, anchor=None):
         """
         Add an image to the sheet.
         Optionally provide a cell for the top-left anchor
         """
         if anchor is not None:
-            cell = self[anchor]
-            img.anchor(cell, anchortype="oneCell")
+            img.anchor = anchor
         self._images.append(img)
 
 
@@ -686,74 +565,61 @@ class Worksheet(_WorkbookChild):
         self._tables.append(table)
 
 
+    def add_pivot(self, pivot):
+        self._pivots.append(pivot)
+
+
     def merge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
+        cr = CellRange(range_string=range_string, min_col=start_column, min_row=start_row,
+                      max_col=end_column, max_row=end_row)
         """ Set merge on a cell range.  Range is a cell range (e.g. A1:E1) """
-        if not range_string and not all((start_row, start_column, end_row, end_column)):
-            msg = "You have to provide a value either for 'coordinate' or for\
-            'start_row', 'start_column', 'end_row' *and* 'end_column'"
-            raise ValueError(msg)
-        elif not range_string:
-            range_string = '%s%s:%s%s' % (get_column_letter(start_column),
-                                          start_row,
-                                          get_column_letter(end_column),
-                                          end_row)
-        elif ":" not in range_string:
-            if COORD_RE.match(range_string):
-                return  # Single cell, do nothing
-            raise ValueError("Range must be a cell range (e.g. A1:E1)")
-        else:
-            range_string = range_string.replace('$', '')
 
-        if range_string not in self._merged_cells:
-            self._merged_cells.append(range_string)
+        self.merged_cells.add(cr.coord)
+        self._clean_merge_range(cr)
+
+
+    def _clean_merge_range(self, cr):
+        """
+        Remove all but the top left-cell from a range of merged cells
+        and creates a MergedCellRange object to recreate the lost border
+        information.
+        After deletion of cells a reformat is issued.
+        """
+
+        min_col, min_row, max_col, max_row = cr.bounds
+
+        mcr = MergedCellRange(self, cr.coord)
+        self._merged_cell_range.update({cr.bounds:mcr})
 
-        min_col, min_row, max_col, max_row = range_boundaries(range_string)
         rows = range(min_row, max_row+1)
         cols = range(min_col, max_col+1)
         cells = product(rows, cols)
-        # all but the top-left cell are removed
-        for c in islice(cells, 1, None):
-            if c in self._cells:
-                del self._cells[c]
 
-
-    @property
-    def merged_cells(self):
-        """Utility for checking whether a cell has been merged or not"""
-        cells = set()
-        for _range in self._merged_cells:
-            for row in rows_from_range(_range):
-                cells = cells.union(set(row))
-        return cells
+        for row, col in islice(cells, 1, None):
+            self._cells[row, col] = MergedCell(self, row, col)
+        mcr.format()
 
 
     @property
+    @deprecated("Use ws.merged_cells.ranges")
     def merged_cell_ranges(self):
         """Return a copy of cell ranges"""
-        return self._merged_cells[:]
+        return self.merged_cells.ranges[:]
 
 
     def unmerge_cells(self, range_string=None, start_row=None, start_column=None, end_row=None, end_column=None):
         """ Remove merge on a cell range.  Range is a cell range (e.g. A1:E1) """
-        if not range_string:
-            if start_row is None or start_column is None or end_row is None or end_column is None:
-                msg = "You have to provide a value either for "\
-                    "'coordinate' or for 'start_row', 'start_column', 'end_row' *and* 'end_column'"
-                raise InsufficientCoordinatesException(msg)
-            else:
-                range_string = '%s%s:%s%s' % (get_column_letter(start_column), start_row, get_column_letter(end_column), end_row)
-        elif len(range_string.split(':')) != 2:
-            msg = "Range must be a cell range (e.g. A1:E1)"
-            raise InsufficientCoordinatesException(msg)
-        else:
-            range_string = range_string.replace('$', '')
+        cr = CellRange(range_string=range_string, min_col=start_column, min_row=start_row,
+                      max_col=end_column, max_row=end_row)
 
-        if range_string in self._merged_cells:
-            self._merged_cells.remove(range_string)
+        if cr.coord not in self.merged_cells:
+            raise ValueError("Cell range {0} is not merged".format(cr.coord))
+
+        self.merged_cells.remove(cr)
+
+        # Deletes the MergedCellRange.
+        del self._merged_cell_range[cr.bounds]
 
-        else:
-            msg = 'Cell range %s not known as merged.' % range_string
-            raise InsufficientCoordinatesException(msg)
 
     def append(self, iterable):
         """Appends a group of values at the bottom of the current sheet.
@@ -784,17 +650,17 @@ class Worksheet(_WorkbookChild):
                     if cell.parent and cell.parent != self:
                         raise ValueError("Cells cannot be copied from other worksheets")
                     cell.parent = self
-                    cell.col_idx = col_idx
+                    cell.column = col_idx
                     cell.row = row_idx
                 else:
-                    cell = Cell(self, row=row_idx, col_idx=col_idx, value=content)
+                    cell = Cell(self, row=row_idx, column=col_idx, value=content)
                 self._cells[(row_idx, col_idx)] = cell
 
         elif isinstance(iterable, dict):
             for col_idx, content in iterable.items():
                 if isinstance(col_idx, basestring):
                     col_idx = column_index_from_string(col_idx)
-                cell = Cell(self, row=row_idx, col_idx=col_idx, value=content)
+                cell = Cell(self, row=row_idx, column=col_idx, value=content)
                 self._cells[(row_idx, col_idx)] = cell
 
         else:
@@ -803,53 +669,132 @@ class Worksheet(_WorkbookChild):
         self._current_row = row_idx
 
 
+    def _move_cells(self, min_row=None, min_col=None, offset=0, row_or_col="row"):
+        """
+        Move either rows or columns around by the offset
+        """
+        reverse = offset > 0 # start at the end if inserting
+        row_offset = 0
+        col_offset = 0
+
+        # need to make affected ranges contiguous
+        if row_or_col == 'row':
+            cells = self.iter_rows(min_row=min_row)
+            row_offset = offset
+            key = 0
+        else:
+            cells = self.iter_cols(min_col=min_col)
+            col_offset = offset
+            key = 1
+        cells = list(cells)
+
+        for row, column in sorted(self._cells, key=itemgetter(key), reverse=reverse):
+            if min_row and row < min_row:
+                continue
+            elif min_col and column < min_col:
+                continue
+
+            self._move_cell(row, column, row_offset, col_offset)
+
+
+    def insert_rows(self, idx, amount=1):
+        """
+        Insert row or rows before row==idx
+        """
+        self._move_cells(min_row=idx, offset=amount, row_or_col="row")
+        self._current_row = self.max_row
+
+
+    def insert_cols(self, idx, amount=1):
+        """
+        Insert column or columns before col==idx
+        """
+        self._move_cells(min_col=idx, offset=amount, row_or_col="column")
+
+
+    def delete_rows(self, idx, amount=1):
+        """
+        Delete row or rows from row==idx
+        """
+
+        remainder = _gutter(idx, amount, self.max_row)
+
+        self._move_cells(min_row=idx+amount, offset=-amount, row_or_col="row")
+
+        for row in remainder:
+            for col in range(self.min_column, self.max_column+1):
+                if (row, col) in self._cells:
+                    del self._cells[row, col]
+        self._current_row = self.max_row
+        if not self._cells:
+            self._current_row = 0
+
+
+    def delete_cols(self, idx, amount=1):
+        """
+        Delete column or columns from col==idx
+        """
+
+        remainder = _gutter(idx, amount, self.max_column)
+
+        self._move_cells(min_col=idx+amount, offset=-amount, row_or_col="column")
+
+        for col in remainder:
+            for row in range(self.min_row, self.max_row+1):
+                if (row, col) in self._cells:
+                    del self._cells[row, col]
+
+
+    def move_range(self, cell_range, rows=0, cols=0, translate=False):
+        """
+        Move a cell range by the number of rows and/or columns:
+        down if rows > 0 and up if rows < 0
+        right if cols > 0 and left if cols < 0
+        Existing cells will be overwritten.
+        Formulae and references will not be updated.
+        """
+        if isinstance(cell_range, basestring):
+            cell_range = CellRange(cell_range)
+        if not isinstance(cell_range, CellRange):
+            raise ValueError("Only CellRange objects can be moved")
+        if not rows and not cols:
+            return
+        min_col, min_row, max_col, max_row = cell_range.bounds
+        # rebase moved range
+        cell_range.shift(row_shift=rows, col_shift=cols)
+
+        down = rows > 0
+        right = cols > 0
+        r = sorted(range(min_row, max_row+1), reverse=down)
+        c = sorted(range(min_col, max_col+1), reverse=right)
+        for row, column in product(r, c):
+            self._move_cell(row, column, rows, cols, translate)
+
+
+    def _move_cell(self, row, column, row_offset, col_offset, translate=False):
+        """
+        Move a cell from one place to another.
+        Delete at old index
+        Rebase coordinate
+        """
+        cell = self._get_cell(row, column)
+        new_row = cell.row + row_offset
+        new_col = cell.column + col_offset
+        self._cells[new_row, new_col] = cell
+        del self._cells[(cell.row, cell.column)]
+        cell.row = new_row
+        cell.column = new_col
+        if translate and cell.data_type == "f":
+            t = Translator(cell.value, cell.coordinate)
+            cell.value = t.translate_formula(row_delta=row_offset, col_delta=col_offset)
+
+
     def _invalid_row(self, iterable):
         raise TypeError('Value must be a list, tuple, range or generator, or a dict. Supplied value is {0}'.format(
             type(iterable))
                         )
 
 
-    @deprecated("Charts and images should be positioned using anchor objects")
-    def point_pos(self, left=0, top=0):
-        """ tells which cell is under the given coordinates (in pixels)
-        counting from the top-left corner of the sheet.
-        Can be used to locate images and charts on the worksheet """
-
-        if left < 0 or top < 0:
-            raise ValueError("Coordinates must be positive")
-
-        current_col = 1
-        current_row = 1
-        column_dimensions = self.column_dimensions
-        row_dimensions = self.row_dimensions
-        default_width = points_to_pixels(DEFAULT_COLUMN_WIDTH)
-        default_height = points_to_pixels(DEFAULT_ROW_HEIGHT)
-        left_pos = 0
-        top_pos = 0
-
-        while left_pos <= left:
-            letter = get_column_letter(current_col)
-            current_col += 1
-            if letter in column_dimensions:
-                cdw = column_dimensions[letter].width
-                if cdw is not None:
-                    left_pos += points_to_pixels(cdw)
-                    continue
-            left_pos += default_width
-
-        while top_pos <= top:
-            row = current_row
-            current_row += 1
-            if row in row_dimensions:
-                rdh = row_dimensions[row].height
-                if rdh is not None:
-                    top_pos += points_to_pixels(rdh)
-                    continue
-            top_pos += default_height
-
-        return (letter, row)
-
-
     def _add_column(self):
         """Dimension factory for column information"""
 
@@ -861,17 +806,9 @@ class Worksheet(_WorkbookChild):
         return RowDimension(self)
 
 
-    def _write(self):
-        from openpyxl.drawing.spreadsheet_drawing import SpreadsheetDrawing
-        from openpyxl.writer.worksheet import write_worksheet
-        self._drawing = SpreadsheetDrawing()
-        self._drawing.charts = self._charts
-        self._drawing.images = self._images
-        return write_worksheet(self)
-
-
     @property
     def print_title_rows(self):
+        """Rows to be printed at the top of every page (ex: '1:3')"""
         if self._print_rows:
             return self._print_rows
 
@@ -890,6 +827,7 @@ class Worksheet(_WorkbookChild):
 
     @property
     def print_title_cols(self):
+        """Columns to be printed at the left side of every page (ex: 'A:C')"""
         if self._print_cols:
             return self._print_cols
 
@@ -917,7 +855,8 @@ class Worksheet(_WorkbookChild):
     @property
     def print_area(self):
         """
-        Return the print area for the worksheet, if set
+        The print area for the worksheet, or None if not set. To set, supply a range
+        like 'A1:D4' or a list of ranges.
         """
         return self._print_area
 
@@ -931,3 +870,13 @@ class Worksheet(_WorkbookChild):
             value = [value]
 
         self._print_area = [absolute_coordinate(v) for v in value]
+
+
+def _gutter(idx, offset, max_val):
+    """
+    When deleting rows and columns are deleted we rely on overwriting.
+    This may not be the case for a large offset on small set of cells:
+    range(cells_to_delete) > range(cell_to_be_moved)
+    """
+    gutter = range(max(max_val+1-offset, idx), min(idx+offset, max_val)+1)
+    return gutter
